C+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
C                                                                              +
C   FUNCTION NAME     - ENORM2                                                 +
C                                                                              +
C+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
C                                                                              +
C   DESCRIPTION:                                                               +
C                                                                              +
C           DOUBLE PRECISION FUNCTION:                                         +
C           COMPUTES THE SQUARE OF THE EUCLIDEAN NORM OF A VECTOR WHICH IS     +
C           A COLUMN OF AN ARRAY.                                              +
C                                                                              +
C   USAGE:                                                                     +
C           ENORM2(A, IA, IR, IP)                                              +
C                                                                              +
C   ARGUMENTS:                                                                 +
C                                                                              +
C           A      - INPUT REAL MATRIX OF DIMENSION (IP,*) CONTAINING          +
C                    A CERTAIN NUMBER OF COLUMN VECTORS.                       +
C                                                                              +
C           IA     - INPUT INTEGER VALUE THAT DEFINES THE NUMBER OF            +
C                    COLUMNS OF THE MATRIX WHOSE NORM HAS TO BE COMPUTED.      +
C                                                                              +
C           IR     - INPUT ROWS DIMENSION EXACTLY AS SPECIFIED IN THE          +
C                    DIMENSION STATEMENTS IN THE CALLING PROGRAM FOR THE       +
C                    MATRIX A.                                                 +
C                                                                              +
C           IP     - INPUT DIMENSION OF THE VECTOR SPACE (NUMBER OF ELEMENTS   +
C                    OF THE VECTOR).                                           +
C                                                                              +
C   REMARKS:                                                                   +
C                                                                              +
C       -  ALL THE REAL ARGUMENTS PASSED FROM THE CALLING PROGRAM MUST BE      +
C          IN DOUBLE PRECISION.                                                +
C       -  IF THE MATRIX A HAS AN UPPER DIMENSION WITH A LOWER-BOUND NOT EQUAL +
C          TO 1, THEN THE USER MUST CONSIDER THAT THE VALUE IA = 1 CORRESPONDS +
C          TO THE FIRST COLUMN OF THE MATRIX, NOT TO THE COLUMN WHOSE SECOND   +
C          COMPONENT IS 1.                                                     +
C       -  IN THE CALLING PROCEDURE THE MINIMAL DIMENSIONS FOR THE             +
C          MATRIX A (IR=IP) ARE:                                               +
C           A(IP,*)                                                            +
C                                                                              +
C+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
C                                                                       
      DOUBLE PRECISION FUNCTION ENORM2(A, IA, IR, IP)
C
C                        ... SPECIFICATIONS FOR ARGUMENTS
C
      DOUBLE PRECISION A(1)
      INTEGER IA, IR, IP
C
C                        ... SPECIFICATIONS FOR LOCAL VARIABLES   
C
      DOUBLE PRECISION ACC
      INTEGER I, IASTART
C
C                        ... EXECUTABLE STATEMENTS
C
      ACC = 0.0D0
      IASTART = (IA-1)*IR 
      DO 10 I = 1, IP
         ACC = ACC + A(IASTART+I)**2
   10 CONTINUE      
C
C                        ... SET THE RESULT VALUE
C
      ENORM2 = ACC 
      RETURN
      END
