C+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
C                                                                              +
C   FUNCTION NAME     - INNERG                                                 +
C                                                                              +
C+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
C                                                                              +
C   DESCRIPTION:                                                               +
C                                                                              +
C           DOUBLE PRECISION FUNCTION:                                         +
C           COMPUTES THE INNER PRODUCT BETWEEN TWO VECTORS WHICH ARE           +
C           A ROW AND A COLUMN OR A PART OF A ROW OR OF A COLUMN OF TWO        +
C           GENERAL ARRAYS.                                                    +
C                                                                              +
C   USAGE:                                                                     +
C           INNERG (A, IAROW, IACOL, IRA, B, IBROW, IBCOL, IRB, IP)            +
C                                                                              +
C   ARGUMENTS:                                                                 +
C                                                                              +
C           A      - INPUT REAL MATRIX WHICH CONTAINS THE ROW VECTOR.          +
C                                                                              +
C           IAROW  - INPUT ROW INDEX OF THE FIRST ELEMENT OF THE ROW VECTOR    +
C                    IN THE MATRIX A.                                          +
C                                                                              +
C           IACOL  - INPUT COLUMN INDEX OF THE FIRST ELEMENT OF THE ROW        +
C                    VECTOR IN THE MATRIX A.                                   +
C                                                                              +
C           IRA    - INPUT ROWS DIMENSION EXACTLY AS SPECIFIED IN THE          +
C                    DIMENSION STATEMENTS IN THE CALLING PROGRAM FOR THE       +
C                    MATRIX A.                                                 +
C                                                                              +
C           B      - INPUT REAL MATRIX WHICH CONTAINS THE COLUMN VECTOR.       +
C                                                                              +
C           IBROW  - INPUT ROW INDEX OF THE FIRST ELEMENT OF THE COLUMN        +
C                    VECTOR IN THE MATRIX B.                                   +
C                                                                              +
C           IBCOL  - INPUT COLUMN INDEX OF THE FIRST ELEMENT OF THE COLUMN     +
C                    VECTOR IN THE MATRIX B.                                   +
C                                                                              +
C           IRB    - INPUT ROWS DIMENSION EXACTLY AS SPECIFIED IN THE          +
C                    DIMENSION STATEMENTS IN THE CALLING PROGRAM FOR THE       +
C                    MATRIX B.                                                 +
C                                                                              +
C           IP     - NUMBER OF ELEMENTS TO BE CONSIDERED IN EACH VECTOR        +
C                    STARTING FROM THE FIRST ONE.                              +
C                                                                              +
C   REMARKS:                                                                   +
C                                                                              +
C        - ALL THE REAL ARGUMENTS PASSED FROM THE CALLING PROGRAM MUST BE      +
C          IN DOUBLE PRECISION.                                                +
C        - IN THE CALLING PROCEDURE, IF THE DIMENSIONS OF THE TWO MATRICES     +
C          A AND B ARE A(IRA,ICA) AND B(IRB,ICB) THEN WE MUST HAVE             +
C          (IAROW.LE. IRA).AND.(IP .LE. ICA).AND.(IP .LE. IRB)                 +
C          .AND.(IBCOL .LE.ICB).                                               +
C                                                                              +
C+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
C                                                                       
      DOUBLE PRECISION FUNCTION INNERG(A, IAROW, IACOL, IRA, 
     *                                 B, IBROW, IBCOL, IRB, IP)
C
C                        ... SPECIFICATIONS FOR ARGUMENTS
C
      DOUBLE PRECISION A(1), B(1) 
      INTEGER IAROW, IACOL, IBROW, IBCOL, IRA, IRB, IP
C
C                        ... SPECIFICATIONS FOR LOCAL VARIABLES   
C
      DOUBLE PRECISION ACC
      INTEGER I, IASTART, IBSTART
C
C                        ... EXECUTABLE STATEMENTS
C
      IASTART = (IACOL - 1) * IRA + IAROW 
      IBSTART = (IBCOL - 1) * IRB + IBROW
      ACC  = 0.0D0
      DO 10 I = 1, IP
         ACC = ACC + A(IASTART+IRA*(I-1)) * B(IBSTART+I-1)
   10 CONTINUE      
C
C                        ... SETS THE RESULT VALUE
C
      INNERG = ACC 
      RETURN
      END
