C+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
C                                                                              +
C   FUNCTION NAME     - INNERP                                                 +
C                                                                              +
C+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
C                                                                              +
C   DESCRIPTION:                                                               +
C                                                                              +
C           DOUBLE PRECISION FUNCTION:                                         +
C           COMPUTES THE INNER PRODUCT BETWEEN TWO VECTORS WHICH ARE           +
C           COLUMNS OF TWO ARRAYS.                                             +
C                                                                              +
C   USAGE:                                                                     +
C           INNERP (A, IA, B, IB, IR, IP)                                      +
C                                                                              +
C   ARGUMENTS:                                                                 +
C                                                                              +
C           A      - INPUT REAL MATRIX OF DIMENSION (IP,*) CONTAINING          +
C                    A CERTAIN NUMBER OF COLUMN VECTORS.                       +
C                                                                              +
C           IA     - INPUT COLUMN OF MATRIX A. FIRST VECTOR OF THE INNER       +
C                    PRODUCT.                                                  +
C                                                                              +
C           B      - INPUT REAL MATRIX OF DIMENSION (IP,*) CONTAINING          +
C                    A CERTAIN NUMBER OF COLUMN VECTORS.                       +
C                                                                              +
C           IB     - INPUT COLUMN OF MATRIX B. SECOND VECTOR OF THE INNER      +
C                    PRODUCT.                                                  +
C                                                                              +
C           IR     - INPUT ROWS DIMENSION EXACTLY AS SPECIFIED IN THE          +
C                    DIMENSION STATEMENTS IN THE CALLING PROGRAM FOR THE       +
C                    MATRICES A AND B.                                         +
C                                                                              +
C           IP     - INPUT DIMENSION OF THE VECTOR SPACE (NUMBER OF ROWS       +
C                    FOR THE MATRICES A AND B).                                +
C                                                                              +
C   REMARKS:                                                                   +
C                                                                              +
C       -  ALL THE REAL ARGUMENTS PASSED FROM THE CALLING PROGRAM MUST BE      +
C          IN DOUBLE PRECISION.                                                +
C       -  IN THE CALLING PROCEDURE, THE ROWS DIMENSION FOR THE MATRICES A     +
C          AND B MUST BE THE SAME.                                             +
C                                                                              +
C+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
C                                                                       
      DOUBLE PRECISION FUNCTION INNERP(A, IA, B, IB, IR, IP)
C
C                        ... SPECIFICATIONS FOR ARGUMENTS
C
      DOUBLE PRECISION A(1), B(1) 
      INTEGER IA, IB, IR, IP
C
C                        ... SPECIFICATIONS FOR LOCAL VARIABLES   
C
      DOUBLE PRECISION ACC
      INTEGER I, IASTART, IBSTART
C
C                        ... EXECUTABLE STATEMENTS
C
      IASTART = (IA-1)*IR 
      IBSTART = (IB-1)*IR 
      ACC  = 0.0D0
      DO 10 I = 1, IP
         ACC = ACC + A(IASTART+I) * B(IBSTART+I)
   10 CONTINUE      
C
C                        ... SET THE RESULT VALUE.
C
      INNERP = ACC 
      RETURN
      END
