C+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
C                                                                              +
C   SUBROUTINE NAME     - SEALGO                                               +
C                                                                              +
C+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
C                                                                              +
C   DESCRIPTION:                                                               +
C                                                                              +
C           SIMULTANEOUS EXTRAPOLATION OF SEVERAL SCALAR SEQUENCES VIA THE     +
C           E-ALGORITHM WITH SEVERAL CHOICES FOR THE AUXILIARY SEQUENCES g .   +
C                                                                         i    +
C                                                                              +
C   USAGE:                                                                     +
C           CALL SEALGO (E, MAXCOL, INIT, EPS, EINIT, GINIT, IR, IP, G,        +
C              IZERO, IER)                                                     +
C                                                                              +
C   ARGUMENTS:                                                                 +
C                                                                              +
C           E      - OUTPUT REAL MATRIX. IN THE CALLING PROCEDURE THE          +
C                    DIMENSIONS OF E MUST BE (IP,0:IE), WITH IE GREATER OR     +
C                    EQUAL TO MAXCOL. THIS MATRIX CONTAINS, AFTER THE k-TH     +
C                    CALL THE LAST COMPUTED ROW OF THE E-ARRAY:                +
C                                                                              +
C                                          (k-1)  (k-2)       (0)              +
C                     - IF k <= MAXCOL+1  E    , E     ,..., E                 +
C                                          0      1           k-1              +
C                                                                              +
C                                          (k-1)  (k-2)       (k-1-MAXCOL)     +
C                     - IF k > MAXCOL+1   E    , E     ,..., E                 +
C                                          0      1           MAXCOL           +
C                    THE DIFFERENT RESULTS ARE STORED AS COLUMNS OF THE        +
C                    MATRIX.                                                   +
C                                                                              +
C           MAXCOL - INPUT INTEGER GIVING THE INDEX OF THE LAST COLUMN         +
C                    OF THE E-ARRAY THAT THE USER WANTS TO COMPUTE             +
C                                         (n)                                  +
C                    (THE LOWER INDEX OF E   ).                                +
C                                         k                                    +
C                                                                              +
C           INIT   - INPUT/OUTPUT INTEGER TO BE SET TO ZERO BEFORE THE FIRST   +
C                    CALL OF THE SUBROUTINE. ITS VALUE IS CHANGED TO 1 BY      +
C                    THE SUBROUTINE DURING THE FIRST CALL.                     +
C                    FOR A NEW APPLICATION OF THE ALGORITHM THE USER           +
C                    MUST SET AGAIN INIT TO ZERO.                              +
C                                                                              +
C           EPS    - INPUT REAL VALUE USED IN TESTS FOR ZERO.                  +
C                    IF DABS(X) .LT. EPS, THEN X IS CONSIDERED TO BE ZERO.     +
C                                                                              +
C           EINIT  - INPUT/OUTPUT REAL VECTOR. IT MUST CONTAIN THE VALUES      +
C                    OF S    BEFORE THE k-TH CALL OF THE SUBROUTINE, THAT      +
C                        k-1                                                   +
C                    IS THE COMPONENTS OF EINIT CONTAIN THE CORRESPONDING      +
C                    TERMS OF THE VARIOUS SEQUENCES TO BE EXTRAPOLATED.        +
C                    IT CONTAINS AFTER THE k-TH CALL OF THE SUBROUTINE         +
C                                                                              +
C                     (0)                                                      +
C                    E            IF  k <= MAXCOL+1                            +
C                     k-1                                                      +
C                                                                              +
C                     (k-1-MAXCOL)                                             +
C                    E            IF  k >  MAXCOL+1                            +
C                     MAXCOL                                                   +
C                                                                              +
C                    THE SUBROUTINE RETURNS THE SEQUENCE                       +
C                                                                              +
C                     (0)   (0)   (0)         (0)      (1)      (2)            +
C                    E   , E   , E   , ... , E      , E      , E      , ...    +
C                     0     1     2           MAXCOL   MAXCOL   MAXCOL         +
C                                                                              +
C           GINIT  - INPUT REAL MATRIX. IN THE CALLING PROCEDURE THE SECOND    +
C                    DIMENSION OF GINIT MUST BE GREATER OR EQUAL TO 2*MAXCOL.  +
C                    BEFORE THE k-TH CALL OF THE SUBROUTINE IT MUST CONTAIN    +
C                    IF k = 1   GINIT(J,1) = G (0)  FOR J = 1, IP              +
C                                             1                                +
C                                                                              +
C                    IF k = 2   GINIT(J,1) = G (1)  FOR J = 1, IP              +
C                                             1                                +
C                                                                              +
C                    IF 3 <= k <= MAXCOL+1                                     +
C                                                                              +
C                       GINIT(J,*)=[G (k-1), G (k-1), ... , G   (k-1),         +
C                                    1        2              k-2               +
C                                   G   (0), G   (1), ... , G   (k-1)]         +
C                                    k-1      k-1            k-1               +
C                       FOR J = 1, IP                                          +
C                                                                              +
C                    IF k > MAXCOL+1                                           +
C                                                                              +
C                       GINIT(J,*)=[G (k-1), G (k-1), ... , G   (k-1)]         +
C                                    1        2              MAXCOL            +
C                       FOR J = 1, IP                                          +
C                                                                              +
C                    EACH G (n) IS A VECTOR WHOSE IP COMPONENTS ARE THE        +
C                          i                                                   +
C                    DIFFERENT POSSIBLE CHOICES FOR THE AUXILIARY SCALAR       +
C                    g (n).                                                    +
C                     i                                                        +
C                                                                              +
C           IR     - INPUT ROWS DIMENSION EXACTLY AS SPECIFIED IN THE          +
C                    DIMENSION STATEMENTS IN THE CALLING PROGRAM FOR THE       +
C                    MATRICES E, GINIT, G, AND FOR THE VECTOR EINIT.           +
C                                                                              +
C           IP     - INPUT VALUE REPRESENTING THE NUMBER OF DIFFERENT          +
C                    SEQUENCES TO BE EXTRAPOLATED AND OF AUXILIARY             +
C                    SEQUENCES g  . (NUMBER OF ROWS FOR THE MATRICES E,        +
C                               i                                              +
C                    GINIT, G, AND OF THE ELEMENTS OF THE VECTORS EINIT        +
C                    AND IZERO).                                               +
C                                                                              +
C           G      - WORKING REAL MATRIX IN WHICH THE LAST ROW AND THE         +
C                    LAST COLUMN OF EACH G  ARRAY ARE SAVED FOR THE NEXT       +
C                                         i                                    +
C                    CALL OF THE SUBROUTINE. IN THE CALLING PROCEDURE THE      +
C                    SECOND DIMENSION OF G MUST BE GREATER OR EQUAL TO         +
C                    MAXCOL*(MAXCOL+1) AND TO MAXCOL*NUMBER_OF_CALLS.          +
C                                                                              +
C           IZERO  - OUTPUT INTEGER VECTOR. EACH COMPONENT IS SET TO ZERO      +
C                    DURING THE FIRST CALL. WHEN A DIVISION BY ZERO IS FOUND   +
C                    IN THE EXECUTION OF THE SUBROUTINE, THE CORRESPONDING     +
C                    ELEMENT IS SET TO ONE AND THE IER VALUE TO 100.           +
C                    AT EACH CALL, THE SUBROUTINE CONSIDERS ONLY THE VALUES    +
C                    OF E, GINIT AND G WITH A ZERO CORRESPONDING ELEMENT IN    +
C                    THE VECTOR IZERO.                                         +
C                                                                              +
C           IER    - INPUT/OUTPUT INDEX WARNING/ERROR.                         +
C                    IER = 100     DIVISION BY ZERO IN THE SUBROUTINE.         +
C                    IER = 200     CALL OF THE SUBROUTINE WITH A NON ZERO      +
C                                  IER VALUE.                                  +
C                    IER = 300     ROWS DIMENSION TOO SMALL IN THE CALLING     +
C                                  PROCEDURE COMPARED TO THE VECTOR SPACE      +
C                                  DIMENSION.                                  +
C                                                                              +
C   REMARKS:                                                                   +
C                                                                              +
C       -  ALL THE REAL ARGUMENTS PASSED FROM THE CALLING PROCEDURE MUST BE    +
C          IN DOUBLE PRECISION. THE WORKING MATRICES E AND G AND THE VECTOR    +
C          IZERO MUST NOT BE MODIFIED BY THE USER BETWEEN TWO CONSECUTIVE      +
C          CALLS OF THE SUBROUTINE.                                            +
C                                  (0)                                         +
C       -  TO OBTAIN THE VALUE OF E   (J <= MAXCOL), THE USER MUST CALL THIS   +
C                                  J                                           +
C          SUBROUTINE (J+1) TIMES.                                             +
C       -  IN THE CALLING PROCEDURE, THE ROWS DIMENSION FOR THE MATRICES E,    +
C          GINIT, G, AND THE DIMENSIONS OF THE VECTOR EINIT MUST BE THE SAME.  +
C       -  WHEN THE SUBROUTINE RETURNS AN IER VALUE OF 100, AT LEAST ONE NEW   +
C          DIVISION BY ZERO HAS BEEN FOUND IN THE SUBROUTINE AND THE RELATED   +
C          ELEMENT(S) OF THE VECTOR IZERO SET(S) TO ONE. THE USER MUST SET     +
C          AGAIN THE VALUE OF IER TO ZERO BEFORE THE NEXT CALL.                +
C       -  THIS SUBROUTINE CAN TRANSFORM SIMULTANEOUSLY SEVERAL SEQUENCES (THE +
C          COMPONENTS OF EINIT) EACH WITH A DIFFERENT FAMILY OF (g , g , ...)  +
C                                                                 1   2        +
C          CORRESPONDING TO THE SAME COMPONENT.                                +
C          IF ALL THE COMPONENTS OF EINIT ARE IDENTICAL THE SUBROUTINE         +
C          TRANSFORMS ONE SINGLE SEQUENCE WITH SEVERAL FAMILIES OF AUXILIARY   +
C          SEQUENCES (g , g , ...).                                            +
C                      1   2                                                   +
C          IF ALL THE ROWS OF GINIT ARE IDENTICAL THE SUBROUTINE TRANSFORMS    +
C          SEVERAL SEQUENCES SIMULTANEOUSLY WITH ONE FAMILY OF AUXILIARY       +
C          SEQUENCES (g , g , ...).                                            +
C                      1   2                                                   +
C       -  IN THE CALLING PROCEDURE THE MINIMAL DIMENSIONS FOR THE             +
C          MATRIX AND VECTOR ARGUMENTS (IR=IP) ARE:                            +
C           E(IP,0:MAXCOL)                                                     +
C           EINIT(IP)                                                          +
C           GINIT(IP,2*MAXCOL)                                                 +
C           G(IP,MAXDIM),MAXDIM=MAX(MAXCOL*(MAXCOL+1),MAXCOL*NUMBER_OF_CALLS)  +
C           IZERO(IP)                                                          +
C                                                                              +
C+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
C                                                                       
      SUBROUTINE SEALGO (E, MAXCOL, INIT, EPS, EINIT, GINIT, IR, 
     *       IP, G, IZERO, IER)
C
C                        ... SPECIFICATIONS FOR ARGUMENTS
C
      INTEGER IER, INIT, IP, IR, MAXCOL
      INTEGER IZERO(1)
      DOUBLE PRECISION EPS
      DOUBLE PRECISION E(IR,0:1), EINIT(1), G(IR,1), GINIT(IR,1)
C
C                        ... SPECIFICATIONS FOR LOCAL VARIABLES   
C
      DOUBLE PRECISION R
      INTEGER I, II, IND, INDM, INDI, INDI1, INDJ, J
      SAVE INDM
C
C                        ... EXECUTABLE STATEMENTS
C
      IF (INIT .EQ. 0) THEN
C
C                        ... FIRST CALL OF THE SUBROUTINE
C
         IF (IP .GT. IR) THEN
            IER = 300
            RETURN
         END IF
         IER    = 0                                                           
         INDM   = 0
         INIT   = 1
         DO 10 II = 1, IP
            E(II,0)    = EINIT(II)
            G(II,1)    = GINIT(II,1)
            IZERO(II)  = 0
   10    CONTINUE
         RETURN
      END IF
C
C                        ... NEXT CALLS OF THE SUBROUTINE               
C
      IF (IER .NE. 0) THEN
         IER = 200
         RETURN
      END IF
C
      IF (MAXCOL .EQ. 0) THEN
         DO 20 II = 1, IP
            E(II,0) = EINIT(II)
   20    CONTINUE
         RETURN
      END IF
C
C                        ... INDM REPRESENTS THE (NUMBER_OF_CALLS - 1)
C
      INDM = INDM+1
C
C                        ... ADD AN ELEMENT TO G  ARRAY AT EACH CALL
C                                               1
C
      DO 30 II = 1, IP
         IF (IZERO(II).EQ.0) G(II,1+INDM*MAXCOL) = GINIT(II,1)
   30 CONTINUE
      IF (INDM .NE. 1) THEN
C
C                        ... COMPUTE AND SAVE IN G THE NECESSARY ELEMENTS 
C                            OF THE G ARRAYS (ONLY FROM THE THIRD CALL)
C                                    i
C
         IF (INDM .GE. 3) THEN
C
C                        ... COMPUTE THE NEXT ROW OF ALL THE PREVIOUS G  
C                                                                      i
C                            ARRAYS (i = 2 TO INDM-1 IF INDM <= MAXCOL+1)
C                                   (i = 2 TO MAXCOL IF INDM > MAXCOL+1)
C                            AND SAVE IN G THE NEW COMPUTED ELEMENTS 
C
            IND = MIN0(INDM-1,MAXCOL)
            DO 60 I = 2, IND
               DO 50 J = 1, I-1
                  INDI = I + (J-1) * MAXCOL
                  INDJ = J + INDM * MAXCOL
                  DO 40 II = 1, IP
                     IF (IZERO(II).EQ.0) THEN
                        R = G(II,INDJ-MAXCOL) - G(II,INDJ)
                        IF (DABS(R) .LT. EPS) THEN
                           IER       = 100
                           IZERO(II) = 1
                        END IF
                     END IF
                     IF (IZERO(II).EQ.0) THEN
                        R = GINIT(II,I) - (G(II,INDI) - GINIT(II,I))/R 
     *                      * G(II,INDJ)
                        G(II,INDI)  = GINIT(II,I)
                        GINIT(II,I) = R
                        IF (J .EQ. (I-1)) G(II,I+INDM*MAXCOL) = R
                     END IF 
   40             CONTINUE
   50          CONTINUE
   60       CONTINUE
         END IF
         IF (INDM .LE. MAXCOL) THEN
C
C                        ... COMPUTE THE NEW G     ARRAY  IF INDM <= MAXCOL
C                                             INDM
C                            AND SAVE IN G THE RELATIVE LAST ROW AND COLUMN
C
            DO 70 II = 1, IP
               IF (IZERO(II).EQ.0) G(II,INDM) = GINIT(II,INDM)
   70       CONTINUE
            DO 100 I= 1, INDM
               INDI1 = INDM + I
               IND   = MIN0(I,INDM-1)
               DO 90 J = 1, IND
                  INDI = J + I * MAXCOL
                  INDJ = INDM + (J-1) * MAXCOL
                  DO 80 II = 1, IP
                     IF (IZERO(II).EQ.0) THEN
                        R = G(II,INDI-MAXCOL) - G(II,INDI)
                        IF (DABS(R) .LT. EPS) THEN
                           IER       = 100
                           IZERO(II) = 1
                        END IF
                     END IF
                     IF (IZERO(II).EQ.0) THEN
                        R = GINIT(II,INDI1) - 
     *                    (G(II,INDJ) - GINIT(II,INDI1))/R * G(II,INDI)
                        G(II,INDJ)      = GINIT(II,INDI1)
                        GINIT(II,INDI1) = R
                        IF (J .EQ. IND) G(II,INDM+I*MAXCOL) = R
                     END IF 
   80             CONTINUE
   90          CONTINUE
  100       CONTINUE
         END IF
      END IF
C
C                        ... COMPUTE THE NEW ELEMENTS OF THE NEXT ROW
C                            IN THE E ARRAY  
C
      IND = MIN0(INDM,MAXCOL)
      DO 120 I = 1, IND
         INDI  = I + INDM*MAXCOL
         DO 110 II = 1, IP
            IF (IZERO(II).EQ.0) THEN
               R = G(II,INDI-MAXCOL) - G(II,INDI) 
               IF (DABS(R) .LT. EPS) THEN
                  IER       = 100
                  IZERO(II) = 1
               END IF
            END IF
            IF (IZERO(II).EQ.0) THEN
               R = EINIT(II) - (E(II,I-1) - EINIT(II))/R 
     *             * G(II,INDI)
               E(II,I-1)  = EINIT(II)
               EINIT(II)  = R
               IF (I .EQ. IND) E(II,IND) = R
            END IF
  110    CONTINUE
C
C                        ... END LOOP FOR I.
C
  120 CONTINUE
      RETURN
      END
